/* Copyright (C) 2005,2006 G.P. Halkes
   Licensed under the Open Software License version 2.0 */

#ifndef OPTION_H
#define OPTION_H

#include "bool.h"
#include "list.h"

#define MAX_VERBOSITY 3

#include "posixregex.h"

#ifdef USE_REGEX
typedef enum {
	DUMP_TOKENS_MULTI = 0,
	DUMP_TOKENS_SEPARATE,
	DUMP_TOKENS_LABELS
} DumpTokensTypes;
#endif

typedef struct {
	/* This flag cannot be set by any command line option, but is set
	   automatically when the program is started by the name LLgen. */
	bool llgenMode;
	
	int verbose;
	bool verboseSet;
	
	/* suppressWarnings suppresses all warnings, suppressWarningsTypes is a
	   bitmap of specific warning options, and unusedIdentifiers is a list of
	   identifiers for which the unused warning is to be suppressed. */
	bool suppressWarnings;
	int suppressWarningsTypes;
	List *unusedIdentifiers;
	
	bool showDir;
	
	bool LLgenArgStyle;
	bool onlyLLgenEscapes;
	
	bool warningsErrors;
	
	const char *outputBaseName;
	bool outputBaseNameSet;
	
	List *inputFileList;
	bool keepDirInFilename;
	bool noPrototypesHeader;
	bool dontGenerateLineDirectives;
	bool LLgenStyleOutputs;
	bool reentrant;
	bool noEOFZero;
	bool noLLreissue;
	bool generateLexerWrapper;
	bool generateLLmessage;
	bool abort;
	bool generateSymbolTable;
	bool noAllowLabelCreate;
	bool noArgCount;
	bool notOnlyReachable;

	int errorLimit;
	bool errorLimitSet;
	
	bool lowercaseSymbols;
	
	bool depend;
	const char *dependTargets;
	const char *dependExtraTargets;
	const char *dependFile;

	bool dependCpp;
	
#ifdef USE_REGEX
	bool useTokenPattern;
	regex_t tokenPattern;
	bool dumpTokens;
	DumpTokensTypes dumpTokensType;
#endif

	bool dumpLLmessage;
	bool dumpLexerwrapper;

	bool threadSafe;
} options;

extern options option;
void parseCmdLine(int argc, char *argv[]);
void postOptionChecks(void);
void checkWarningSuppressionArguments(const char *optarg, int optlength, const char *current_option);
#endif
